@@ -4,6 +4,8 @@
 #ifndef VECGEOM_BASE_TRANSFORMATION3D_H_
 #define VECGEOM_BASE_TRANSFORMATION3D_H_
 
+#include "VecGeom/base/Config.h"
+#include "VecGeom/base/Cuda.h"
 #include "VecGeom/base/Global.h"
 
 #include "VecGeom/base/Vector3D.h"
@@ -169,7 +171,7 @@ public:
     fRotation[5] *= sz;
     fRotation[6] *= sx;
     fRotation[7] *= sy;
-    fRotation[8] *= sz;    
+    fRotation[8] *= sz;
   }
 
   VECCORE_ATT_HOST_DEVICE
@@ -219,7 +221,7 @@ public:
     Precision sx = std::sqrt(fRotation[0]*fRotation[0] + fRotation[3]*fRotation[3] + fRotation[6]*fRotation[6]);
     Precision sy = std::sqrt(fRotation[1]*fRotation[1] + fRotation[4]*fRotation[4] + fRotation[7]*fRotation[7]);
     Precision sz = std::sqrt(fRotation[2]*fRotation[2] + fRotation[5]*fRotation[5] + fRotation[8]*fRotation[8]);
-    
+
     if (Determinant() < 0) sz = -sz;
 
     return Vector3D<Precision>(sx, sy, sz);
@@ -289,25 +291,23 @@ public:
                    const Precision rot4, const Precision rot5, const Precision rot6, const Precision rot7,
                    const Precision rot8);
 
+  /**
+   * Set transformation and rotation.
+   * \param trans Pointer to at least 3 values.
+   * \param rot Pointer to at least 9 values.
+   */
   VECCORE_ATT_HOST_DEVICE
   VECGEOM_FORCE_INLINE
   void Set(const Precision *trans, const Precision *rot, bool has_trans, bool has_rot)
   {
-    // Memory copy translation and rotation components.
-    // Avoid memcpy that gives warning: non-null argument 2 expected
-    // Avoid copy by de-referencing trans/rot which may not have alignof(Precision)
-    auto trans_src  = reinterpret_cast<const char *>(trans);
-    auto trans_dest = reinterpret_cast<char *>(fTranslation);
-    auto rot_src    = reinterpret_cast<const char *>(rot);
-    auto rot_dest   = reinterpret_cast<char *>(fRotation);
     if (has_trans) {
-      for (size_t i = 0; i < 3 * sizeof(Precision); ++i)
-        trans_dest[i] = trans_src[i];
+      for (size_t i = 0; i < 3; ++i)
+        fTranslation[i] = trans[i];
     }
 
     if (has_rot) {
-      for (size_t i = 0; i < 9 * sizeof(Precision); ++i)
-        rot_dest[i] = rot_src[i];
+      for (size_t i = 0; i < 9; ++i)
+        fRotation[i] = rot[i];
     }
 
     fHasTranslation = has_trans;
