@@ -4,6 +4,8 @@
 #ifndef VECGEOM_MANAGEMENT_BVHMANAGER_H_
 #define VECGEOM_MANAGEMENT_BVHMANAGER_H_
 
+#include "VecGeom/base/Config.h"
+#include "VecGeom/base/Cuda.h"
 #include "VecGeom/base/BVH.h"
 #include "VecGeom/volumes/LogicalVolume.h"
 
@@ -14,16 +16,10 @@
 namespace vecgeom {
 inline namespace VECGEOM_IMPL_NAMESPACE {
 inline std::vector<BVH *> hBVH;
-} // namespace VECGEOM_IMPL_NAMESPACE
-
 #ifdef VECGEOM_ENABLE_CUDA
-namespace cuda {
 inline __device__ BVH *dBVH;
-}
 #endif
 
-inline namespace VECGEOM_IMPL_NAMESPACE {
-
 /**
  * @brief The @c BVHManager class is a singleton class to manage the association between
  * logical volumes and their bounding volume hierarchies, using the logical volumes' ids.
