@@ -5,6 +5,7 @@
 #ifndef VECGEOM_NAVIGATION_NAVSTATEINDEX_H_
 #define VECGEOM_NAVIGATION_NAVSTATEINDEX_H_
 
+#include "VecGeom/base/Config.h"
 #include "VecGeom/base/Global.h"
 #include "VecGeom/base/Transformation3D.h"
 #include "VecGeom/volumes/PlacedVolume.h"
@@ -245,10 +246,18 @@ public:
     }
 
     if ((hasm & 0x03) == 0) return;
-    bool has_trans           = (hasm & 0x02) > 0;
-    bool has_rot             = (hasm & 0x01) > 0;
-    auto nd                  = GetNdaughtersImpl(nav_ind);
-    const Precision *address = (Precision *)(NavIndAddr(nav_ind + 3 + nd + ((nd + 1) & 1)));
+    bool has_trans = (hasm & 0x02) > 0;
+    bool has_rot   = (hasm & 0x01) > 0;
+    auto nd        = GetNdaughtersImpl(nav_ind);
+
+    // Potentially skip one NavIndex_t to ensure alignment of transformation data
+    auto transformationDataIndex     = nav_ind + 3 + nd + ((nd + 1) & 1);
+    const bool padTransformationData = (transformationDataIndex * sizeof(NavIndex_t)) % sizeof(::Precision) != 0;
+    transformationDataIndex += unsigned{padTransformationData};
+
+    const auto address = reinterpret_cast<const Precision *>(NavIndAddr(transformationDataIndex));
+    assert(reinterpret_cast<uintptr_t>(address) % sizeof(Precision) == 0);
+
     Transformation3D t;
     t.Set(address, address + 3, has_trans, has_rot);
     t.MultiplyFromRight(trans);
