@@ -4,8 +4,6 @@
 #include "VecGeom/base/BVH.h"
 
 #include "VecGeom/management/ABBoxManager.h"
-#include "VecGeom/volumes/LogicalVolume.h"
-#include "VecGeom/volumes/PlacedVolume.h"
 
 #include <algorithm>
 #include <cmath>
@@ -17,11 +15,11 @@ inline namespace VECGEOM_IMPL_NAMESPACE {
 
 constexpr int BVH::BVH_MAX_DEPTH;
 
-static int ClosestAxis(Vector3D<Precision> v)
-{
-  v = v.Abs();
-  return v[0] > v[2] ? (v[0] > v[1] ? 0 : 1) : (v[1] > v[2] ? 1 : 2);
-}
+enum class BVH::ConstructionAlgorithm : unsigned int {
+  SplitLongestAxis = 0,
+  LargestDistanceAlongAxis = 1,
+  SurfaceAreaHeuristic = 2,
+};
 
 /**
  * \class BVH
@@ -77,9 +75,11 @@ BVH::BVH(LogicalVolume const &volume, int depth) : fLV(volume)
   fNChild = new int[nodes];
   fOffset = new int[nodes];
   fNodes  = new AABB[nodes];
+  std::fill(fNChild, fNChild+nodes, 0);
+  std::fill(fOffset, fOffset+nodes, -1);
 
   /* Recursively initialize BVH nodes starting at the root node */
-  ComputeNodes(0, fPrimId, fPrimId + n, nodes);
+  ComputeNodes(0, fPrimId, fPrimId + n, nodes, ConstructionAlgorithm::SurfaceAreaHeuristic);
 
   /* Mark internal nodes with a negative number of children to simplify traversal */
   for (unsigned int id = 0; id < nodes / 2; ++id)
@@ -95,10 +95,26 @@ BVH::BVH(LogicalVolume const *volume, int depth, int *dPrimId, AABB *dAABBs, int
 #endif
 
 VECCORE_ATT_HOST_DEVICE
-void BVH::Print() const
+void BVH::Print(bool verbose) const
 {
-  printf("BVH(%u): addr: %p, depth: %d, nodes: %d, children: %lu\n", fLV.id(), this, fDepth, (2 << fDepth) - 1,
-         fLV.GetDaughters().size());
+  printf("\nBVH(%u): addr: %p, depth: %d, nodes: %d, children: %zu, name: %s\n",
+         fLV.id(), this, fDepth, (2 << fDepth) - 1, fLV.GetDaughters().size(),
+         fLV.GetName() );
+  if (verbose) {
+    constexpr auto width = 4;
+    int nChildToPad = 1;
+    for (int depth = fDepth; depth >= 0; --depth) {
+      const auto begin = (1 << depth) - 1;
+      const auto end   = (2 << depth) - 1;
+      for (int node = begin; node < end; ++node) {
+        if (nChildToPad > 1) printf("%*c", (nChildToPad-1)*width/2, ' ');
+        printf("%3d ", fNChild[node]);
+        if (nChildToPad > 1) printf("%*c", (nChildToPad-1)*width/2, ' ');
+      }
+      printf("\n");
+      nChildToPad *= 2;
+    }
+  }
 }
 
 #ifdef VECGEOM_CUDA_INTERFACE
@@ -114,21 +130,21 @@ DevicePtr<cuda::BVH> BVH::CopyToGpu(void *addr) const
 
   int n = fLV.GetDaughters().size();
 
-  CudaCheckError(cudaMalloc((void **)&dPrimId, n * sizeof(int)));
-  CudaCheckError(cudaMalloc((void **)&dAABBs, n * sizeof(AABB)));
+  CudaCheckError(CudaMalloc((void **)&dPrimId, n * sizeof(int)));
+  CudaCheckError(CudaMalloc((void **)&dAABBs, n * sizeof(AABB)));
 
-  CudaCheckError(cudaMemcpy((void *)dPrimId, (void *)fPrimId, n * sizeof(int), cudaMemcpyHostToDevice));
-  CudaCheckError(cudaMemcpy((void *)dAABBs, (void *)fAABBs, n * sizeof(AABB), cudaMemcpyHostToDevice));
+  CudaCheckError(CudaCopyToDevice((void *)dPrimId, (void *)fPrimId, n * sizeof(int)));
+  CudaCheckError(CudaCopyToDevice((void *)dAABBs, (void *)fAABBs, n * sizeof(AABB)));
 
   int nodes = (2 << fDepth) - 1;
 
-  CudaCheckError(cudaMalloc((void **)&dOffset, nodes * sizeof(int)));
-  CudaCheckError(cudaMalloc((void **)&dNChild, nodes * sizeof(int)));
-  CudaCheckError(cudaMalloc((void **)&dNodes, nodes * sizeof(AABB)));
+  CudaCheckError(CudaMalloc((void **)&dOffset, nodes * sizeof(int)));
+  CudaCheckError(CudaMalloc((void **)&dNChild, nodes * sizeof(int)));
+  CudaCheckError(CudaMalloc((void **)&dNodes, nodes * sizeof(AABB)));
 
-  CudaCheckError(cudaMemcpy((void *)dOffset, (void *)fOffset, nodes * sizeof(int), cudaMemcpyHostToDevice));
-  CudaCheckError(cudaMemcpy((void *)dNChild, (void *)fNChild, nodes * sizeof(int), cudaMemcpyHostToDevice));
-  CudaCheckError(cudaMemcpy((void *)dNodes, (void *)fNodes, nodes * sizeof(AABB), cudaMemcpyHostToDevice));
+  CudaCheckError(CudaCopyToDevice((void *)dOffset, (void *)fOffset, nodes * sizeof(int)));
+  CudaCheckError(CudaCopyToDevice((void *)dNChild, (void *)fNChild, nodes * sizeof(int)));
+  CudaCheckError(CudaCopyToDevice((void *)dNodes, (void *)fNodes, nodes * sizeof(AABB)));
 
   cuda::LogicalVolume const *dvolume = CudaManager::Instance().LookupLogical(&fLV).GetPtr();
 
@@ -153,6 +169,179 @@ BVH::~BVH()
 #endif
 }
 
+namespace {
+int ClosestAxis(Vector3D<Precision> v)
+{
+  v = v.Abs();
+  return v[0] > v[2] ? (v[0] > v[1] ? 0 : 1) : (v[1] > v[2] ? 1 : 2);
+}
+
+int * splitAlongLongestAxis(const AABB * primitiveBoxes,
+                            int * begin, int * end,
+                            const AABB & currentBVHNode) {
+  const Vector3D<Precision> basis[] = {{1.0, 0.0, 0.0}, {0.0, 1.0, 0.0}, {0.0, 0.0, 1.0}};
+  Vector3D<Precision> p = currentBVHNode.Center();
+  Vector3D<Precision> v = basis[ClosestAxis(currentBVHNode.Size())];
+
+  return std::partition(begin, end, [&](size_t i) { return Vector3D<Precision>::Dot(primitiveBoxes[i].Center() - p, v) < 0.0; });
+}
+
+int * largestDistanceAlongAxis(const AABB * primitiveBoxes,
+                               int * begin, int * end,
+                               const AABB & /*currentBVHNode*/) {
+  // Compute maximum extension of lower-left front corners along all axes
+  float extension[3][2] = {{0.f, 0.f}, {0.f, 0.f}, {0.f, 0.f}};
+  for (int axis = 0; axis <= 2; ++axis) {
+    auto minMaxIt = std::minmax_element(begin, end, [=](size_t a, size_t b){
+      return primitiveBoxes[a].Min()[axis] < primitiveBoxes[b].Min()[axis];
+    });
+    extension[axis][0] = primitiveBoxes[*minMaxIt.first].Min()[axis];
+    extension[axis][1] = primitiveBoxes[*minMaxIt.second].Min()[axis];
+  }
+
+  const int splitAxis = std::distance(extension, std::max_element(extension, extension+3, [](float a[], float b[]){
+    return a[1]-a[0] < b[1]-b[0];
+  }));
+  const float middlePoint = (extension[splitAxis][1] + extension[splitAxis][0]) / 2.;
+  return std::partition(begin, end, [=](size_t i) {
+    return primitiveBoxes[i].Min()[splitAxis] < middlePoint;
+  });
+}
+
+/**
+ * In order to achieve stable splitting for the BVH, we cannot only sort by one axis. There needs
+ * to be a strict order, so elements are not silently considered equal by the STL algorithms.
+ * @param left,right Compute `left < right`.
+ * @param sortAxis Principal axis to sort by.
+ */
+template<typename T>
+bool less3D(const T & left, const T & right, const int sortAxis) {
+  return left[sortAxis] < right[sortAxis] ||
+    ( left[sortAxis] == right[sortAxis] &&
+      ( left[(sortAxis+1)%3] < right[(sortAxis+1)%3] ||
+        ( left[(sortAxis+1)%3] == right[(sortAxis+1)%3] && left[(sortAxis+2)%3] < right[(sortAxis+2)%3] )
+      )
+    );
+}
+
+/**
+ * Compute the surface areas of bounding boxes that surround the given primitives,
+ * sweeping from left to right and vice-versa.
+ *
+ * For three objects 0 1 2, the vector contains the following surface areas:
+ * ( | 0+1+2)  (0 | 1+2)   (0+1 | 2)
+ *
+ * That is, if the object N is intended to be the pivot object, the surface area of
+ * - everything left of N is `areas[N].first`
+ * - everything right of N + N itself is `areas[N].second`
+ *
+ * @param primitiveBoxes Array of bounding boxes of primitives.
+ * @param begin Index of first primitive to be considered.
+ * @param end   Past-the-end index of primitives to be considered.
+ */
+std::vector<std::pair<double, double>> sweepSurfaceArea(const AABB * primitiveBoxes, int const * begin, int const * end) {
+  if (begin >= end) return {};
+
+  std::vector<std::pair<double, double>> areas(std::distance(begin, end), {0., 0.});
+
+  AABB box{primitiveBoxes[*begin]};
+  for (auto it = begin+1; it < end; ++it) {
+    areas[it-begin].first = box.SurfaceArea();
+    box = AABB::Union(box, primitiveBoxes[*it]);
+  }
+
+  AABB box2{primitiveBoxes[*(end-1)]};
+  for (auto it = end - 1; it >= begin; --it) {
+    box2 = AABB::Union(box2, primitiveBoxes[*(it)]);
+    areas[it-begin].second = box2.SurfaceArea();
+  }
+
+  return areas;
+}
+
+/**
+ * Use the surface area heuristic to construct a BVH tree.
+ * This algorithm tries to split the primitives such that they form clusters that have a minimal surface
+ * area, as this decreases the likelihood that a BVH node is intersected by a ray.
+ * Contrary to what's used in standard graphics, the cost function has an additional term that prevents
+ * very large clusters. For the conventional SAH, a long line of equally spaced primitives would does not
+ * yield an obvious splitting point, as all splits lead to the same total surface area for both child nodes.
+ * To prevent this, there is an extra term, which will encourage a 50:50 split.
+ * @param primitveBoxes Array of bounding boxes of primitives.
+ * @param begin Index of first primitive to be considered.
+ * @param end   Past-the-end index of primitives to be considered.
+ * @return Index of the first element of the second group. If this is `end`, no good split was found.
+ */
+int * surfaceAreaHeuristic(const AABB * primitiveBoxes,
+                          int * begin, int * end,
+                          const AABB & /*currentBVHNode*/) {
+  int bestSplitAxis = -1;
+  double bestTraversalMetric = std::distance(begin, end);
+  int bestSplitObject = -1;
+  const auto nObj = std::distance(begin, end);
+
+  int currentSortAxis = 0;
+  auto sorter = [primitiveBoxes, &currentSortAxis](int a, int b) {
+    const auto centroidA = primitiveBoxes[a].Center();
+    const auto centroidB = primitiveBoxes[b].Center();
+    constexpr double shift = 0.01;
+    return less3D(centroidA + shift*(centroidA - primitiveBoxes[a].Min()),
+                  centroidB + shift*(centroidB - primitiveBoxes[b].Min()),
+                  currentSortAxis);
+  };
+
+  for (int axis = 0; axis <= 2; ++axis) {
+    // Sort centroids along axis
+    currentSortAxis = axis;
+    std::sort(begin, end, sorter);
+
+    // Sweep axis looking for best split
+    const std::vector<std::pair<double,double>> surfaceSweep = sweepSurfaceArea(primitiveBoxes, begin, end);
+    const auto totSurfArea = surfaceSweep.front().second;
+
+    for (int * splitObject = begin; splitObject < end; ++splitObject) {
+      const auto left  = surfaceSweep[splitObject-begin].first/totSurfArea;
+      const auto right = surfaceSweep[splitObject-begin].second/totSurfArea;
+      assert(left <= 1. && right <= 1.);
+
+      const auto splitMetric =
+            left * std::distance(begin, splitObject)
+          + right * std::distance(splitObject, end)
+          + 0.1 * abs(nObj/2 - std::distance(begin, splitObject) / nObj); // Prefer balanced splits
+
+      if (splitMetric < bestTraversalMetric) {
+        bestTraversalMetric = splitMetric;
+        bestSplitAxis = axis;
+        bestSplitObject = *splitObject;
+      }
+    }
+  }
+
+  if (bestSplitAxis == -1)
+    return end;
+
+  currentSortAxis = bestSplitAxis;
+  auto result = std::partition(begin, end, [sorter,bestSplitObject](size_t i) {
+    return sorter(i, bestSplitObject);
+  });
+
+  return result;
+}
+
+/**
+ * Array of splitting functions that can be used to construct the BVH tree.
+ * @see BVH::ConstructionAlgorithm
+ */
+int * (*splittingFunction[])(const AABB * /*primitveAABBs*/,
+                            int * /*firstPrimitive*/, int * /*lastPrimitive*/,
+                            const AABB & /*currentBVHNode*/) = {
+  &splitAlongLongestAxis,
+  &largestDistanceAlongAxis,
+  &surfaceAreaHeuristic,
+};
+
+} // anonymous namespace
+
 /*
  * BVH::ComputeNodes() initializes nodes of the BVH. It first computes the number of children that
  * belong to the current node based on the iterator range that is passed as input, as well as the
@@ -167,101 +356,37 @@ BVH::~BVH()
  * same side of the splitting plane), or if the node contains only a single volume.
  */
 
-void BVH::ComputeNodes(unsigned int id, int *first, int *last, unsigned int nodes)
+void BVH::ComputeNodes(unsigned int id, int *first, int *last, unsigned int nodes,
+                       BVH::ConstructionAlgorithm constructionAlgorithm)
 {
-  const Vector3D<Precision> basis[] = {{1.0, 0.0, 0.0}, {0.0, 1.0, 0.0}, {0.0, 0.0, 1.0}};
-
-  if (id >= nodes || first == last) return;
+  if (id >= nodes) return;
 
   fNChild[id] = std::distance(first, last);
   fOffset[id] = std::distance(fPrimId, first);
 
+  // Node without children. Stop recursing here.
+  if (first == last) return;
+
   fNodes[id] = fAABBs[*first];
   for (auto it = std::next(first); it != last; ++it)
     fNodes[id] = AABB::Union(fNodes[id], fAABBs[*it]);
 
+  // Only one child. No need to continue
   if (std::next(first) == last) return;
 
-  Vector3D<Precision> p = fNodes[id].Center();
-  Vector3D<Precision> v = basis[ClosestAxis(fNodes[id].Size())];
+  const auto algo = static_cast<unsigned int>(constructionAlgorithm);
+  assert(algo < sizeof(splittingFunction));
 
-  auto mid =
-      std::partition(first, last, [&](size_t i) { return Vector3D<Precision>::Dot(fAABBs[i].Center() - p, v) < 0.0; });
+  int * pivot = splittingFunction[algo](fAABBs, first, last, fNodes[id]);
+  assert(first <= pivot && pivot <= last);
 
-  ComputeNodes(2 * id + 1, first, mid, nodes);
-  ComputeNodes(2 * id + 2, mid, last, nodes);
-}
-
-/*
- * BVH::ComputeDaughterIntersections() computes the intersection of a ray against all children of
- * the logical volume. A stack is kept of the node ids that need to be checked. It needs to be at
- * most as deep as the binary tree itself because we always first pop the current node, and then
- * add at most the two children. For example, for depth two, we pop the root node, then at most we
- * add both of its leaves onto the stack to be checked. We initialize ptr with &stack[1] such that
- * when we pop the first time as we enter the loop, the position we read from is the first position
- * of the stack, which contains the id 0 for the root node. When we pop the stack such that ptr
- * points before &stack[0], it means we've checked all we needed and the loop can be terminated.
- * In order to determine if a node of the tree is internal or not, we check if the node id of its
- * left child is past the end of the array (in which case we know we are at the maximum depth), or
- * if the sum of children in both leaves is the same as in the current node, as for leaf nodes, the
- * sum of children in the left+right child nodes will be less than for the current node.
- */
-
-VECCORE_ATT_HOST_DEVICE
-void BVH::CheckDaughterIntersections(Vector3D<Precision> localpoint, Vector3D<Precision> localdir, Precision &step,
-                                     VPlacedVolume const *last, VPlacedVolume const *&hitcandidate) const
-{
-  unsigned int stack[BVH_MAX_DEPTH] = {0}, *ptr = &stack[1];
-
-  /* Calculate and reuse inverse direction to save on divisions */
-  Vector3D<Precision> invdir(1.0 / NonZero(localdir[0]), 1.0 / NonZero(localdir[1]), 1.0 / NonZero(localdir[2]));
-
-  do {
-    unsigned int id = *--ptr; /* pop next node id to be checked from the stack */
-
-    if (fNChild[id] >= 0) {
-      /* For leaf nodes, loop over children */
-      for (int i = 0; i < fNChild[id]; ++i) {
-        int prim = fPrimId[fOffset[id] + i];
-        /* Check AABB first, then the volume itself if needed */
-        if (fAABBs[prim].IntersectInvDir(localpoint, invdir, step)) {
-          auto vol  = fLV.GetDaughters()[prim];
-          auto dist = vol->DistanceToIn(localpoint, localdir, step);
-          /* If distance to current child is smaller than current step, update step and hitcandidate */
-          if (dist < step && !(dist <= 0.0 && vol == last)) step = dist, hitcandidate = vol;
-        }
-      }
-    } else {
-      unsigned int childL = 2 * id + 1;
-      unsigned int childR = 2 * id + 2;
-
-      /* For internal nodes, check AABBs to know if we need to traverse left and right children */
-      Precision tminL = kInfLength, tmaxL = -kInfLength, tminR = kInfLength, tmaxR = -kInfLength;
-
-      fNodes[childL].ComputeIntersectionInvDir(localpoint, invdir, tminL, tmaxL);
-      fNodes[childR].ComputeIntersectionInvDir(localpoint, invdir, tminR, tmaxR);
-
-      bool traverseL = tminL <= tmaxL && tmaxL >= 0.0 && tminL < step;
-      bool traverseR = tminR <= tmaxR && tmaxR >= 0.0 && tminR < step;
-
-      /*
-       * If both left and right nodes need to be checked, check closest one first.
-       * This ensures step gets short as fast as possible so we can skip more nodes without checking.
-       */
-      if (tminR < tminL) {
-        if (traverseR) *ptr++ = childR;
-        if (traverseL) *ptr++ = childL;
-      } else {
-        if (traverseL) *ptr++ = childL;
-        if (traverseR) *ptr++ = childR;
-      }
-    }
-  } while (ptr > stack);
+  ComputeNodes(2 * id + 1, first, pivot, nodes, constructionAlgorithm);
+  ComputeNodes(2 * id + 2, pivot, last,  nodes, constructionAlgorithm);
 }
 
 /*
  * BVH::ApproachNextDaughter is very similar to CheckDaughterIntersections but computes the first
- * hit daughter bounding box instead of the next hit shape. This lighter computation is used to 
+ * hit daughter bounding box instead of the next hit shape. This lighter computation is used to
  * first approach the next hit solid before computing the actual distance, in the attempt to
  * reduce the numerical rounding error due to propagation to boundary.
  */
@@ -323,150 +448,6 @@ void BVH::ApproachNextDaughter(Vector3D<Precision> point, Vector3D<Precision> di
   } while (ptr > stack);
 }
 
-/*
- * BVH::ComputeSafety is very similar to the method above regarding traversal of the tree, but it
- * computes only the safety instead of the intersection using a ray, so the logic is a bit simpler.
- */
-
-VECCORE_ATT_HOST_DEVICE
-Precision BVH::ComputeSafety(Vector3D<Precision> localpoint, Precision safety) const
-{
-  unsigned int stack[BVH_MAX_DEPTH] = {0}, *ptr = &stack[1];
-
-  do {
-    unsigned int id = *--ptr;
-
-    if (fNChild[id] >= 0) {
-      for (int i = 0; i < fNChild[id]; ++i) {
-        int prim = fPrimId[fOffset[id] + i];
-        if (fAABBs[prim].Safety(localpoint) < safety) {
-          Precision dist = fLV.GetDaughters()[prim]->SafetyToIn(localpoint);
-          if (dist < safety) safety = dist;
-        }
-      }
-    } else {
-      unsigned int childL = 2 * id + 1;
-      unsigned int childR = 2 * id + 2;
-
-      Precision safetyL = fNodes[childL].Safety(localpoint);
-      Precision safetyR = fNodes[childR].Safety(localpoint);
-
-      bool traverseL = safetyL < safety;
-      bool traverseR = safetyR < safety;
-
-      if (safetyR < safetyL) {
-        if (traverseR) *ptr++ = childR;
-        if (traverseL) *ptr++ = childL;
-      } else {
-        if (traverseL) *ptr++ = childL;
-        if (traverseR) *ptr++ = childR;
-      }
-    }
-  } while (ptr > stack);
-
-  return safety;
-}
-
-VECCORE_ATT_HOST_DEVICE
-bool BVH::LevelLocate(Vector3D<Precision> const &localpoint, VPlacedVolume const *&pvol,
-                      Vector3D<Precision> &daughterlocalpoint) const
-{
-  VPlacedVolume const *exclvol = nullptr;
-  return LevelLocate(exclvol, localpoint, pvol, daughterlocalpoint);
-}
-
-VECCORE_ATT_HOST_DEVICE
-bool BVH::LevelLocate(Vector3D<Precision> const &localpoint, NavigationState &state,
-                      Vector3D<Precision> &daughterlocalpoint) const
-{
-  VPlacedVolume const *exclvol = nullptr;
-  VPlacedVolume const *pvol = nullptr;
-  bool Result = LevelLocate(exclvol, localpoint, pvol, daughterlocalpoint);
-  if (Result) {
-    state.Push(pvol);
-  }
-  return Result;
-}
-
-VECCORE_ATT_HOST_DEVICE
-bool BVH::LevelLocate(VPlacedVolume const *exclvol, Vector3D<Precision> const &localpoint, VPlacedVolume const *&pvol,
-                      Vector3D<Precision> &daughterlocalpoint) const
-{
-  unsigned int stack[BVH_MAX_DEPTH] = {0}, *ptr = &stack[1];
-
-  do {
-    unsigned int id = *--ptr;
-
-    if (fNChild[id] >= 0) {
-      for (int i = 0; i < fNChild[id]; ++i) {
-        int prim = fPrimId[fOffset[id] + i];
-        if (fAABBs[prim].Contains(localpoint)) {
-          auto vol = fLV.GetDaughters()[prim];
-          if (vol != exclvol && vol->Contains(localpoint, daughterlocalpoint)) {
-            pvol = vol;
-            return true;
-          }
-        }
-      }
-    } else {
-      unsigned int childL = 2 * id + 1;
-      if (fNodes[childL].Contains(localpoint)) *ptr++ = childL;
-
-      unsigned int childR = 2 * id + 2;
-      if (fNodes[childR].Contains(localpoint)) *ptr++ = childR;
-    }
-  } while (ptr > stack);
-
-  return false;
-}
-
-VECCORE_ATT_HOST_DEVICE
-bool BVH::LevelLocate(VPlacedVolume const *exclvol, Vector3D<Precision> const &localpoint,
-                      Vector3D<Precision> const &localdirection, VPlacedVolume const *&pvol,
-                      Vector3D<Precision> &daughterlocalpoint) const
-{
-  unsigned int stack[BVH_MAX_DEPTH] = {0}, *ptr = &stack[1];
-
-  do {
-    unsigned int id = *--ptr;
-
-    if (fNChild[id] >= 0) {
-      for (int i = 0; i < fNChild[id]; ++i) {
-        int prim = fPrimId[fOffset[id] + i];
-        if (fAABBs[prim].Contains(localpoint)) {
-          auto v = fLV.GetDaughters()[prim];
-
-          if (v == exclvol) continue;
-
-          const auto T = v->GetTransformation();
-          const auto u = v->GetUnplacedVolume();
-          const auto p = T->Transform(localpoint);
-
-          auto Entering = [&]() {
-            Vector3D<Precision> normal, dir = T->TransformDirection(localdirection);
-            u->Normal(p, normal);
-            return Vector3D<Precision>::Dot(normal, dir) < 0.0;
-          };
-
-          auto inside = u->Inside(p);
-
-          if (inside == kInside || (inside == kSurface && Entering())) {
-            pvol = v, daughterlocalpoint = p;
-            return true;
-          }
-        }
-      }
-    } else {
-      unsigned int childL = 2 * id + 1;
-      if (fNodes[childL].Contains(localpoint)) *ptr++ = childL;
-
-      unsigned int childR = 2 * id + 2;
-      if (fNodes[childR].Contains(localpoint)) *ptr++ = childR;
-    }
-  } while (ptr > stack);
-
-  return false;
-}
 
 } // namespace VECGEOM_IMPL_NAMESPACE
 
