@@ -616,6 +616,69 @@ DevicePtr<cuda::VUnplacedVolume> UnplacedPolyhedron::CopyToGpu() const
   return CopyToGpuImpl<UnplacedPolyhedron>();
 }
 
+/**
+ * Bulk-copy UnplacedPolyhedron instances to the device.
+ * This function is significantly faster in constructing unplaced polyhedra on the GPU, because it bulk
+ * copies the constructor arguments to the device, saving a lot of memory allocations and kernel invocations.
+ * @param volumes Pointers to UnplacedPolyhedron instances.
+ * @param devicePointer Locations where the unplaced polyhedra should be constructed.
+ */
+void UnplacedPolyhedron::CopyToGpu(std::vector<VUnplacedVolume const *> const & volumes,
+                                   std::vector<DevicePtr<cuda::VUnplacedVolume>> const & devicePointers)
+{
+  const auto size = volumes.size();
+  std::vector<Precision> floatData(2*size, 0.);
+  std::vector<int> intData(2*size, 0);
+
+  struct VarLengthData {
+    std::vector<std::size_t> offsets[3];
+    std::vector<Precision>   data;
+  } vld;
+  struct RAIIDevPtr {
+    DevicePtr<Precision> devPtr;
+    RAIIDevPtr() = default;
+    RAIIDevPtr(const RAIIDevPtr&) = delete;
+    ~RAIIDevPtr() { devPtr.Deallocate(); }
+  } vldGPU;
+
+  for (unsigned int i = 0; i < size; ++i) {
+    UnplacedPolyhedron const & volume = static_cast<UnplacedPolyhedron const &>(*volumes[i]);
+    floatData[0*size + i] = volume.fPoly.fPhiStart;
+    floatData[1*size + i] = volume.fPoly.fPhiDelta;
+
+    intData[0*size + i] = volume.fPoly.fSideCount;
+    intData[1*size + i] = volume.fPoly.fZPlanes.size();
+
+    int offsetCounter = 0;
+    for (vecgeom::cxx::Array<Precision> const * array : {&volume.fPoly.fZPlanes, &volume.fPoly.fRMin, &volume.fPoly.fRMax}) {
+      vld.offsets[offsetCounter++].push_back(vld.data.size());
+      for (int j = 0; j < array->size(); ++j) {
+        vld.data.push_back((*array)[j]);
+      }
+    }
+  }
+
+  vldGPU.devPtr.Allocate(vld.data.size());
+  vldGPU.devPtr.ToDevice(vld.data.data(), vld.data.size());
+
+  std::vector<Precision const *> zPlanesGpuPtr, rMinGpuPtr, rMaxGpuPtr;
+  auto computeGpuPtr = [&vldGPU](std::size_t offset){ return vldGPU.devPtr.GetPtr() + offset; };
+  std::transform(vld.offsets[0].begin(), vld.offsets[0].end(), std::back_inserter(zPlanesGpuPtr), computeGpuPtr);
+  std::transform(vld.offsets[1].begin(), vld.offsets[1].end(), std::back_inserter(rMinGpuPtr), computeGpuPtr);
+  std::transform(vld.offsets[2].begin(), vld.offsets[2].end(), std::back_inserter(rMaxGpuPtr), computeGpuPtr);
+
+
+  // Forwards all data to this constructor:
+  // UnplacedPolyhedron(Precision phiStart, Precision phiDelta, const int sideCount,
+  //                     const int zPlaneCount, Precision const zPlanes[], Precision const rMin[],
+  //                     Precision const rMax[])
+  ConstructManyOnGpu<vecgeom::cuda::UnplacedPolyhedron>(size, devicePointers.data(),
+      floatData.data(), floatData.data()+size, // phiStart, phiDelta
+      intData.data(), intData.data()+size, // sideCount, zPlaneCount
+      zPlanesGpuPtr.data(), rMinGpuPtr.data(), rMaxGpuPtr.data()
+      );
+}
+
 #endif
 
 } // namespace VECGEOM_IMPL_NAMESPACE
@@ -629,6 +692,10 @@ template void DevicePtr<cuda::UnplacedPolyhedron>::Construct(Precision phiStart,
                                                              int zPlaneCount, DevicePtr<Precision> zPlanes,
                                                              DevicePtr<Precision> rMin,
                                                              DevicePtr<Precision> rMax) const;
+template void ConstructManyOnGpu<cuda::UnplacedPolyhedron>(std::size_t nElement, DevicePtr<cuda::VUnplacedVolume> const * gpu_ptrs,
+                                                           Precision const * phiStart, Precision const * phiDelta,
+                                                           int const * sideCount, int const * zPlaneCount,
+                                                           Precision const * const * zPlanes, Precision const * const * rMin, Precision const * const * rMax);
 
 } // namespace cxx
 
